///////////////////////////////////////////////////////////////////////////////
// Cylinder class
//
// Last update: 29/06/2005
///////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////
// Native JAVA imports
///////////////////////////////////////////////////////////////////////////////
import java.awt.Color;

///////////////////////////////////////////////////////////////////////////////
// JavaView imports
///////////////////////////////////////////////////////////////////////////////
import jv.function.PuFunction;
import jv.geom.PgElementSet;
import jv.geom.PgPolygon;
import jv.geom.PgPolygonSet;
import jv.vecmath.PdVector;
import jv.vecmath.PiVector;
import jv.number.PuDouble;
import jv.project.PvCameraIf;
import jv.project.PvDisplayIf;
import jv.project.PjProject;
import jvx.surface.PgDomainDescr;
import jvx.surface.PgParmSurface;


///////////////////////////////////////////////////////////////////////////////
// Cylinder applet
///////////////////////////////////////////////////////////////////////////////
public class PjCylinder extends PjProject
{
   protected int idiom;
   
   protected PuDouble t;   

   protected double uMin = -Math.PI;
   protected double uMax = +Math.PI;
   protected int nu = 100;
   
   protected double hMin = -1.5;
   protected double hMax = +1.5;
   protected int nh = 20;
   
   protected PuFunction m_surFunction;       
   protected PgDomainDescr m_surDomain;
   protected PgParmSurface m_surParmSurface;
   
   protected double tLev = 0.05;
   
   protected int nc = 100;
   protected PgPolygon curve;
   
   protected PvCameraIf cam;
   protected PvDisplayIf disp;
   
   protected PgElementSet plane;
   protected boolean isShowingPlane = true;
   
   protected PgPolygonSet axes1;
   protected boolean isShowingAxes = false;
   

   //////////////////////////////////////////////////////////////////////////
   // Default constructor
   //////////////////////////////////////////////////////////////////////////
   public PjCylinder(int input_idiom)
   {
      super("Cylinder Applet");
      
      idiom = input_idiom;

      if (getClass() == PjCylinder.class)
      {
         init();
      };
   }; // End of PjCylinder()


   //////////////////////////////////////////////////////////////////////////
   // Initialize variables
   //////////////////////////////////////////////////////////////////////////
   public void init()
   {
       // t slider
       t = new PuDouble("t:", this);     
       t.setDefBounds(0.0, 1.0, 0.01, 0.05);
       t.setDefValue(0);
       t.init();
       t.setEnabled(true); 
       t.setEnabledConfigButton(false);
       
       // Surface
       m_surFunction = new PuFunction(2, 3);
       m_surFunction.setName("Surface Function");
       m_surFunction.setExpression(0, "sin(u)");
       m_surFunction.setExpression(1, "-cos(u) + 1");
       m_surFunction.setExpression(2, "v");

       m_surDomain = new PgDomainDescr(2);
       m_surDomain.setName("Surface Domain");
       m_surDomain.setMaxSize(uMin, hMin, uMax, hMax);
       m_surDomain.setSize(uMin, hMin, uMax, hMax);
       m_surDomain.setDiscr(nu, nh);
       m_surDomain.init();

       m_surParmSurface = new PgParmSurface(3);
       m_surParmSurface.setDomainDescr(m_surDomain);
       m_surParmSurface.setFunctionExpr(m_surFunction);
       m_surParmSurface.compute();

       m_surParmSurface.showTransparency(true);
       m_surParmSurface.setTransparency(tLev);   
       m_surParmSurface.showEdges(false);
       
       // Curve
       curve = new PgPolygon(3); // This is a 3D object
       curve.setName("Curve");
       curve.setGlobalEdgeColor(Color.red);
       curve.showVertices(false);
       curve.setNumVertices(nc + 1);
       
       for (int i = 0; i <= nc; i++)
       {
    	   double theta = -Math.PI + (2.0*Math.PI*i)/nc;
    	   double x = Math.sin(theta);
    	   double y = -Math.cos(theta) + 1;
    	   double z = -Math.sin(theta);
    	   curve.setVertex(i, x, y, z);    	   
       }
       
       // Plane
       plane = new PgElementSet(3);
       plane.setNumVertices(4);
       plane.setNumElements(1);
       
       double l = -1.5;
       PiVector idx = new PiVector(4);
       idx.setEntry(0, 0);
       idx.setEntry(1, 1);
       idx.setEntry(2, 2);
       idx.setEntry(3, 3);
       
       plane.setVertex(0, +l, +l + 1, -l);
       plane.setVertex(1, -l, +l + 1, +l);
       plane.setVertex(2, -l, -l + 1, +l);
       plane.setVertex(3, +l, -l + 1, -l);
       plane.setElement(0, idx);
       
       plane.setGlobalElementColor(Color.RED);
       plane.showTransparency(true);
       plane.setTransparency(0.8);
       
       // Axes
       // Axes
       // Axes 1 configuration
       axes1 = new PgPolygonSet(3); // This is a 3D object
       axes1.setNumVertices(4);
       axes1.setNumPolygons(2);       
       axes1.setVertex(0, new PdVector(-Math.PI - 0.5, -0.005, 0.0));
       axes1.setVertex(1, new PdVector(0.0, -0.005, hMin - 0.5));
       axes1.setVertex(2, new PdVector(+Math.PI + 0.5, -0.005,   0.0));
       axes1.setVertex(3, new PdVector(0.0, -0.005, hMax + 0.5));
       axes1.getVertex(0).setName("");
       axes1.getVertex(1).setName("");
       axes1.getVertex(2).setName("");
       axes1.getVertex(3).setName("");
       axes1.showVertexLabels(true);
       axes1.setPolygon(0, 2, 0);
       axes1.setPolygon(1, 1, 3);
       axes1.setGlobalPolygonColor(Color.BLUE);
       axes1.setGlobalVertexColor(Color.BLUE);
       axes1.setGlobalVertexSize(0.0);       
       axes1.setPolygonSize(0, 0.5);
       axes1.setPolygonSize(1, 0.5);
       axes1.showPolygonSizes(true);
       axes1.showPolygonEndArrow(true);       
   }; // End of init()


   //////////////////////////////////////////////////////////////////////////
   // Things to do when the class is started
   //////////////////////////////////////////////////////////////////////////
   public void start()
   {	
	  addGeometry(curve);
	  addGeometry(plane);
	  addGeometry(m_surParmSurface);
	  
	  disp = getDisplay();	  
	  cam = disp.getCamera();	  	  
      updateCamera();
	  
      update(this);
      
      updateCamera();
   }; // End of start()

   
   //////////////////////////////////////////////////////////////////////////
   // Update surface
   //////////////////////////////////////////////////////////////////////////
   public void updateScene()
   {
	   // t
	   double tVal = t.getValue();
	   
	   if (tVal > 0.999)
	   {
		   tVal = 0.999;
	   };
	   
	   if (tVal > 0.0)
	   {
		   if (isShowingPlane == true)
		   {
		      disp.removeGeometry(plane);
		      isShowingPlane = false;
		   }
	   }
	   else
	   {
		   disp.addGeometry(plane);
		   isShowingPlane = true;
	   }
	   
	   if (t.getValue() < 1.0)
	   {
		   if (isShowingAxes == true)
		   {
		      disp.removeGeometry(axes1);
		      isShowingAxes = false;
		   }
	   }
	   else
	   {
		   disp.addGeometry(axes1);
		   isShowingAxes = true;
	   }
	   
	   
	   // R
	   double R = Math.tan(tVal*Math.PI/2.0) + 1.0;
	   String R_str = "(" + Double.toString(R) + ")";
		   
       // Surface
       m_surFunction = new PuFunction(2, 3);
       m_surFunction.setName("Surface Function");
       m_surFunction.setExpression(0, R_str + "*sin(u/" + R_str +")");
       m_surFunction.setExpression(1, "-" + R_str + "*cos(u/" + R_str +") + " + R_str);
       m_surFunction.setExpression(2, "v");

       m_surDomain = new PgDomainDescr(2);
       m_surDomain.setName("Surface Domain");
       m_surDomain.setMaxSize(uMin, hMin, uMax, hMax);
       m_surDomain.setSize(uMin, hMin, uMax, hMax);
       m_surDomain.setDiscr(nu, nh);
       m_surDomain.init();

       m_surParmSurface.setDomainDescr(m_surDomain);
       m_surParmSurface.setFunctionExpr(m_surFunction);
       m_surParmSurface.compute();

       m_surParmSurface.update(m_surParmSurface);
       

       // Curve
       for (int i = 0; i <= nc; i++)
       {
    	   double theta = -Math.PI + (2.0*Math.PI*i)/nc;
    	   if (theta/(Math.PI) > 0.999)
    	   {
    		   theta = 0.999*Math.PI;
    	   }
    		
    	   double x = R*Math.sin(theta/R);
    	   double y = -R*Math.cos(theta/R) + R;
    	   double z = -Math.sin(theta);
    	   curve.setVertex(i, x, y, z);    	   
       }
       
       curve.update(curve);       
   }; // End of updateScene()

   //////////////////////////////////////////////////////////////////////////
   // Update surface
   //////////////////////////////////////////////////////////////////////////   
   public void updateCamera()
   {
	  if (cam != null)
	  {				  
         cam.setInterest(new PdVector(0, 0, 0));
         cam.setPosition(new PdVector(0, 10, 0));
         cam.setDist(7.0);
         cam.setRoll(0);
         cam.update(cam);  	
         // disp.fit();
         cam.setDist(7.0);
         cam.update(cam);
	 disp.center();
	  }	  	       
   }
   
   /////////////////////////////////////////////////////////////////////////
   // Update the class whenever a child has changed. Method is usually
   // invoked from the children.
   /////////////////////////////////////////////////////////////////////////
   public boolean update(Object event)
   {
      if (event == null)
      {
         return(true);
      };
      
      if (event == t)
      {
          updateScene();

          return(true);
      };
      
      return(super.update(event));
   }; // End of update()
}; // End of PjCylinder classs
